"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2022-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
"{ Package: 'jv:libgdbs' }"

"{ NameSpace: Smalltalk }"

GDBArchitecture subclass:#GDBArch_x86
	instanceVariableNames:'mode disassembler buffer'
	classVariableNames:''
	poolDictionaries:''
	category:'GDB'
!

!GDBArch_x86 class methodsFor:'documentation'!

copyright
"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2022-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
! !

!GDBArch_x86 class methodsFor:'initialization'!

initialize
    "Invoked at system start or when the class is dynamically loaded."

     Smalltalk isSmalltalkX ifTrue:[
        (Smalltalk includesKey:#UDIS86) ifFalse:[
            PackageLoadError ignoreIn:[
                Smalltalk loadPackage: 'jv:dragonfly/udis86sx'.
            ]
        ].
    ].

    "/ please change as required (and remove this comment)

    "Modified: / 30-10-2018 / 19:39:05 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBArch_x86 methodsFor:'accessing'!

name
    ^ mode == 64 ifTrue:[ 'x86_64' ] ifFalse:[ 'i386' ]

    "Created: / 16-08-2018 / 07:37:36 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 16-08-2018 / 11:01:25 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

registerNameOfProgramCounter
    ^ mode == 64 ifTrue: [ 'rip' ] ifFalse: [ 'eip' ]

    "Created: / 09-12-2022 / 12:48:56 / Jan Vrany <jan.vrany@labware.com>"
! !

!GDBArch_x86 methodsFor:'initialization'!

setMode: anInteger
    self assert: (anInteger == 32 or:[ anInteger == 64 ]).
    mode := anInteger.
    (Smalltalk includesKey:#UDIS86) ifTrue:[
        disassembler := (Smalltalk at:#UDIS86) new.
        disassembler mode: mode.
        buffer := ExternalBytes unprotectedNew: 16
    ].

    "Created: / 16-08-2018 / 11:00:33 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 20-08-2018 / 07:53:22 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBArch_x86 methodsFor:'queries'!

sizeofPointer
    ^ 8

    "Created: / 16-08-2018 / 09:35:51 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBArch_x86 methodsFor:'utilities'!

disassemble1: bytes pc: pc
    "
    Disassemble one instruction from given bytes (as ByteArray)
    and return the instruction dissection. Returned object must
    conform to GDBInstructionDissection protocol.

    @see GDBInstructionDissection
    "
    disassembler isNil ifTrue:[ 
        "/ UDIS86 not available.
        ^ super disassemble1: bytes pc: pc
    ].
    buffer replaceBytesFrom: 1  to: bytes size with: bytes startingAt: 1.
    disassembler buffer: buffer pc: pc.
    ^ disassembler disassemble

    "Created: / 16-08-2018 / 11:07:17 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 16-08-2018 / 14:00:23 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBArch_x86 class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !


GDBArch_x86 initialize!
