"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2022-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
"{ Package: 'jv:libgdbs' }"

"{ NameSpace: Smalltalk }"

GDBObject subclass:#GDBCommandResult
	instanceVariableNames:'command status value'
	classVariableNames:''
	poolDictionaries:'GDBCommandStatus'
	category:'GDB-Commands'
!

!GDBCommandResult class methodsFor:'documentation'!

copyright
"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2022-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
! !

!GDBCommandResult methodsFor:'accessing'!

command
    ^ command
!

command:aGDBCommand
    command := aGDBCommand.
!

status
    ^ status
!

status:statusValue
    status := statusValue.
!

value
    "Returns a command result value as GDB object or nil, if
     command result is just a set of properties. In that case,
     use #propertyAt: to query individual property values."
    ^ value

    "Modified (comment): / 19-03-2015 / 08:19:25 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBCommandResult methodsFor:'accessing-properties'!

properties
    ^ GDBObject getPropertiesOf: self.

    "Modified: / 20-06-2014 / 09:04:23 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

propertyAt: name

    "/ We shoud ignore `status` or `command` instvars 
    "/ these are used for other purposes here.
    (name ~= #value and:[self class instVarNames includes: name]) ifTrue:[ 
        properties isNil ifTrue:[ ^ nil ].
        ^ properties at: name ifAbsent: [ nil ].
    ]. 
    ^ GDBObject getProperty: name of: self

    "Created: / 31-05-2014 / 00:00:33 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 30-12-2018 / 20:01:23 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

propertyAt: name put: val

    "/ We shoud not set `status` or `command` instvars even if
    "/ command response contains 'property' with that names -
    "/ these are used for other purposes here.
    (name ~= #value and:[self class instVarNames includes: name]) ifTrue:[ 
        properties isNil ifTrue:[ 
            properties := Dictionary new.
            properties at: name put: val.
        ].
        ^ self
    ].
    ^ GDBObject setProperty: name of: self to: val

    "Created: / 31-05-2014 / 00:01:10 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 30-12-2018 / 20:01:33 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBCommandResult methodsFor:'printing & storing'!

printOn:aStream
    "append a printed representation of the receiver to the argument, aStream"

    super printOn:aStream.
    aStream 
        nextPut:$(;
        nextPutAll: status;
        nextPutAll:': ';
        nextPutAll: command asString;
        nextPut:$).

    "Modified: / 19-01-2019 / 22:58:12 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBCommandResult methodsFor:'testing'!

isDone
    ^ status = CommandStatusDone

    "Created: / 12-06-2017 / 09:36:46 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

isError
    ^ status = CommandStatusError

    "Created: / 12-06-2017 / 09:36:53 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBCommandResult class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

