"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2022-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
"{ Package: 'jv:libgdbs' }"

"{ NameSpace: Smalltalk }"

GDBObject subclass:#GDBDebuggerObject
	instanceVariableNames:'debugger'
	classVariableNames:''
	poolDictionaries:''
	category:'GDB'
!

!GDBDebuggerObject class methodsFor:'documentation'!

copyright
"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2022-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
! !

!GDBDebuggerObject class methodsFor:'testing'!

isAbstract
    ^ self == GDBDebuggerObject
! !

!GDBDebuggerObject methodsFor:'accessing'!

debugger
    ^ debugger
! !

!GDBDebuggerObject methodsFor:'finalization'!

finalizationLobby
    self assert: debugger notNil.
    ^ debugger finalizationLobby.

    "Created: / 28-01-2018 / 23:21:52 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 14-07-2023 / 13:37:57 / Jan Vrany <jan.vrany@labware.com>"
!

finalizationRegistry
    "For Pharo compatibility"
    ^ self finalizationLobby

    "Created: / 14-07-2023 / 13:38:22 / Jan Vrany <jan.vrany@labware.com>"
!

registerForFinalization
    Smalltalk isSmalltalkX ifTrue: [ 
        super registerForFinalization.
        ^ self.
    ].
    Smalltalk isPharo ifTrue: [ 
        self finalizationRegistry add: self.
        ^ self.
    ].
    self error: 'Unsupported dialect'

    "Created: / 14-07-2023 / 13:40:51 / Jan Vrany <jan.vrany@labware.com>"
! !

!GDBDebuggerObject methodsFor:'initialization & release'!

setDebugger: aGDBDebugger
    self assert: (debugger isNil or:[ aGDBDebugger isNil or:[debugger == aGDBDebugger]]).
    debugger := aGDBDebugger

    "Created: / 07-09-2014 / 21:17:43 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 07-02-2019 / 15:15:49 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBDebuggerObject methodsFor:'private'!

ensureIsValid
    self isValid ifFalse:[
        (GDBInvalidObjectError newException)
            parameter:self;
            messageText:'Object is no longer valid';
            raise.
    ].

    "Created: / 04-02-2018 / 21:27:49 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

updateFrom: anotherObject
    "Update receiver  from `anotherObject`. This can be (is) used to update
     existing objects from 'new' ones in order to preseve identity of objects."

    | names ignored |

    self assert: self class == anotherObject class.
    self assert: (debugger == anotherObject debugger or:[anotherObject debugger isNil]).

    names := self class allInstVarNames.
    ignored := self updateFromIgnoredInstvarNames collect:[ :name | names indexOf: name ].
    1 to: self class instSize do:[:index | 
        (ignored includes: index) ifFalse:[
            self instVarAt: index put: (anotherObject instVarAt: index).
        ]. 
    ].

    "Created: / 25-02-2019 / 15:04:13 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 25-02-2019 / 17:37:48 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

updateFromIgnoredInstvarNames
    "Return an array of instvar names to be ignored (skipped) when
     updating old object with 'new' one. See #updateFrom:.

     When overriding, *always* concatenate with super, i.e.,

        updateFromIgnoredInstvarNames
            ^ super updateFromIgnoredInstvarNames , #(some instvar)

    "
    ^ #(debugger)

    "Created: / 25-02-2019 / 15:06:44 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 25-02-2019 / 16:27:38 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBDebuggerObject methodsFor:'testing'!

isValid
    "Return `true`, if the object is still `valid`, false otherwise. For example,
     a `GDBFrame` instance may become invalid when the frame is unwound and no
     longer exist."

    ^ self subclassResponsibility

    "Created: / 04-02-2018 / 21:24:20 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBDebuggerObject class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

