"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2021-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
"{ Package: 'jv:libgdbs' }"

"{ NameSpace: Smalltalk }"

Announcement subclass:#GDBEvent
	instanceVariableNames:'debugger token properties'
	classVariableNames:''
	poolDictionaries:''
	category:'GDB-Events'
!

!GDBEvent class methodsFor:'documentation'!

copyright
"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2021-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
! !

!GDBEvent class methodsFor:'accessing-magritte'!

descriptionContainer
    ^ GDBMAContainer forClass: self.

    "Created: / 23-09-2014 / 23:26:24 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBEvent class methodsFor:'testing'!

isAbstract
    ^ self == GDBEvent

    "Modified: / 02-06-2014 / 22:20:26 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBEvent methodsFor:'accessing'!

debugger
    ^ debugger
!

debugger:aGDBDebugger
    debugger := aGDBDebugger.
!

token
    ^ token
!

token:anInteger
    token := anInteger.
! !

!GDBEvent methodsFor:'accessing-properties'!

properties
    ^ GDBObject getPropertiesOf: self.

    "Modified: / 20-06-2014 / 09:04:23 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

propertyAt: name
    ^ GDBObject getProperty: name of: self

    "Created: / 31-05-2014 / 00:00:33 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 20-06-2014 / 09:05:05 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

propertyAt: name put: value
    ^ GDBObject setProperty: name of: self to: value

    "Created: / 31-05-2014 / 00:01:10 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 20-06-2014 / 09:05:34 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBEvent methodsFor:'printing & storing'!

printDetailOn: aStream
    "Nothing special, to be overriden by subclasses"

    "Created: / 22-02-2022 / 11:52:38 / Jan Vrany <jan.vrany@labware.com>"
!

printOn:aStream
    "append a printed representation of the receiver to the argument, aStream"

    super printOn:aStream.
    aStream nextPut: $(.
    token notNil ifTrue: [ 
        token printOn: aStream base: 10 size: 3 fill: Character space.
    ] ifFalse: [ 
        aStream nextPutAll: '???'.
    ].
    aStream nextPutAll: ', '.
    self printDetailOn: aStream.
    aStream nextPut: $).

    "Created: / 22-02-2022 / 11:52:11 / Jan Vrany <jan.vrany@labware.com>"
! !

!GDBEvent methodsFor:'private - dispatching'!

dispatchUsing: aGDBEventDispatcher
    "Dispatch the event using given event dispatcher.

     STRICTLY PRIVATE"

    aGDBEventDispatcher dispatchEvent: self.

    "Created: / 29-03-2021 / 12:21:30 / Jan Vrany <jan.vrany@labware.com>"
! !

!GDBEvent methodsFor:'testing'!

isAsyncEvent
    "return false here; to be redefined in subclass(es)"

    ^ false
!

isCommandResultEvent
    ^ false
!

isConsoleOutputEvent
    ^ false

    "Created: / 01-06-2014 / 23:37:39 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

isEventSetProcessingFinishedEvent
    ^ false

    "Created: / 22-05-2021 / 15:50:48 / Jan Vrany <jan.vrany@labware.com>"
!

isEventSetProcessingStartedEvent
    ^ false

    "Created: / 22-05-2021 / 15:50:38 / Jan Vrany <jan.vrany@labware.com>"
!

isExecutionEvent
    ^ false

    "Created: / 01-06-2014 / 23:38:03 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

isLogOutputEvent
    ^ false

    "Created: / 01-06-2014 / 23:37:44 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

isNotificationEvent
    ^ false

    "Created: / 01-06-2014 / 23:37:55 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

isStatusEvent
    ^ false

    "Created: / 01-06-2014 / 23:38:07 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

isStopEvent
    ^ false

    "Created: / 10-09-2021 / 13:56:02 / Jan Vrany <jan.vrany@labware.com>"
!

isTargetOutputEvent
    ^ false

    "Created: / 01-06-2014 / 23:37:48 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBEvent class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

