"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2020 LabWare
Copyright (C) 2022-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
"{ Package: 'jv:libgdbs' }"

"{ NameSpace: Smalltalk }"

ApplicationModel subclass:#GDBMIDebugger
	instanceVariableNames:'command process cliView miView'
	classVariableNames:''
	poolDictionaries:''
	category:'GDB-Internal-MI Trace'
!

!GDBMIDebugger class methodsFor:'documentation'!

copyright
"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2020 LabWare
Copyright (C) 2022-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
! !

!GDBMIDebugger class methodsFor:'interface specs'!

windowSpec
    "This resource specification was automatically generated
     by the UIPainter of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the UIPainter may not be able to read the specification."

    "
     UIPainter new openOnClass:GDBMIDebugger andSelector:#windowSpec
     GDBMIDebugger new openInterface:#windowSpec
     GDBMIDebugger open
    "

    <resource: #canvas>

    ^ 
    #(FullSpec
       name: windowSpec
       window: 
      (WindowSpec
         label: 'GDB/MI Debugger / Viewer'
         name: 'GDB/MI Debugger / Viewer'
         uuid: '75dbc6f1-9274-11e9-b1d9-606720e43e2c'
         min: (Point 10 10)
         bounds: (Rectangle 0 0 852 533)
         menu: mainMenu
      ) component: 
      (SpecCollection
         collection: (
          (VariableVerticalPanelSpec
             name: 'VariableVerticalPanel1'
             layout: (LayoutFrame 0 0 0 0 0 1 0 1)
             component: 
            (SpecCollection
               collection: (
                (ArbitraryComponentSpec
                   name: 'CLI'
                   hasBorder: false
                   component: cliView
                )
                (ArbitraryComponentSpec
                   name: 'MI'
                   hasBorder: false
                   component: miView
                )
              )
            ) handles: 
            (Any
               0.5 1.0
            )
          )
        )
      )
    )
! !

!GDBMIDebugger class methodsFor:'menu specs'!

mainMenu
    "This resource specification was automatically generated
     by the MenuEditor of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the MenuEditor may not be able to read the specification."


    "
     MenuEditor new openOnClass:GDBMIDebugger andSelector:#mainMenu
     (Menu new fromLiteralArrayEncoding:(GDBMIDebugger mainMenu)) startUp
    "

    <resource: #menu>

    ^ 
     #(Menu (
         (MenuItem
            label: 'Debug'
            startGroup: right
            isVisible: true
            submenu: 
           (Menu (
               (MenuItem
                  label: 'Open & Attach to GDB Process'
                  itemValue: doDebugOpenAndAttachToGDB
                  isVisible: true
               )
             ) nil
              nil
           )
         )
       ) nil
        nil
     )
! !

!GDBMIDebugger class methodsFor:'utilities'!

debug: command
    "
    Open GDB/MI Debugger on given GDB executable
    "
    self new
        command: command;
        open

    "Created: / 23-06-2020 / 14:12:22 / Jan Vrany <jan.vrany@labware.com>"
! !

!GDBMIDebugger methodsFor:'accessing'!

command
    ^ command
!

command:aString
    self assert: aString asFilename exists.
    command := aString.

    "Modified: / 23-06-2020 / 14:13:42 / Jan Vrany <jan.vrany@labware.com>"
! !

!GDBMIDebugger methodsFor:'aspects'!

cliView
    cliView isNil ifTrue:[ 
        cliView := VT100TerminalView new.
        self initializeTerminalView: cliView.  
    ].
    ^ cliView

    "Created: / 19-06-2019 / 10:33:07 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

miView
    miView isNil ifTrue:[ 
        miView := VT100TerminalView new.
        self initializeTerminalView: miView.  
    ].
    ^ miView

    "Created: / 19-06-2019 / 10:34:36 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBMIDebugger methodsFor:'hooks'!

commonPostOpen
    super commonPostOpen.

    process := GDBLocalProcess newWithCommand: command.  

    (process isKindOf: GDBStXUnixProcess) ifTrue:[ 
        (process instVarNamed: #debuggerPTY) 
            setLocalEcho: true;
            setOutputCRLF: true.
    ].
    miView inStream: process debuggerInput.
    miView outStream: process debuggerOutput. 
    miView startReaderProcessWhenVisible.

    process consoleInput notNil ifTrue:[ 
        cliView inStream: process consoleInput.
        cliView outStream: process consoleOutput. 
        cliView startReaderProcessWhenVisible.    
    ] ifFalse:[ 
        cliView hide.
    ].

    "Created: / 19-06-2019 / 10:40:36 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 23-06-2020 / 14:13:54 / Jan Vrany <jan.vrany@labware.com>"
! !

!GDBMIDebugger methodsFor:'initialization & release'!

initializeTerminalView: aTerminalView
    aTerminalView cursorType: #block;  
                foregroundColor: Color white
                backgroundColor: Color black;
                cursorForegroundColor: Color white
                      backgroundColor: Color white.      
    (Smalltalk at: #VDBAbstractApplication) notNil ifTrue:[ 
        aTerminalView font: (Smalltalk at: #VDBAbstractApplication) defaultTextFont
    ].

    "Created: / 19-06-2019 / 10:33:51 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

release
    process notNil ifTrue:[ 
        process release.
    ].

    "Created: / 19-06-2019 / 10:47:01 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBMIDebugger methodsFor:'menu actions'!

doDebugOpenAndAttachToGDB
    | debuggerOnGDB |

    (Smalltalk includesKey: #VDBDebuggerApplication) ifFalse: [ 
         Smalltalk loadPackage: #'jv:vdb'.
    ].

    debuggerOnGDB := GDBDebugger new.
    (Smalltalk at: #VDBDebuggerApplication) openFor: debuggerOnGDB.
    debuggerOnGDB attach: process id.
! !

!GDBMIDebugger class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

