"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2021-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
"{ Package: 'jv:libgdbs' }"

"{ NameSpace: Smalltalk }"

GDBMICommand subclass:#GDBMI_data_write_memory_bytes
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'GDB-Commands-MI'
!

!GDBMI_data_write_memory_bytes class methodsFor:'documentation'!

copyright
"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2021-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
!

documentation
"
The `-data-write-memory-bytes' Command
--------------------------------------

Synopsis
........

      -data-write-memory-bytes ADDRESS CONTENTS
      -data-write-memory-bytes ADDRESS CONTENTS [COUNT]

where:

`ADDRESS'
     An expression specifying the address of the first memory word to be
     read.  Complex expressions containing embedded white space should
     be quoted using the C convention.

`CONTENTS'
     The hex-encoded bytes to write.

`COUNT'
     Optional argument indicating the number of bytes to be written.
     If COUNT is greater than CONTENTS' length, {No value for `GDBN'}
     will repeatedly write CONTENTS until it fills COUNT bytes.


{No value for `GDBN'} Command
.............................

There's no corresponding {No value for `GDBN'} command.

Example
.......

     (gdb)
     -data-write-memory-bytes &a 'aabbccdd'
     ^done
     (gdb)

     (gdb)
     -data-write-memory-bytes &a 'aabbccdd' 16e
     ^done
     (gdb)


"
! !

!GDBMI_data_write_memory_bytes methodsFor:'accessing'!

operation
	^ 'data-write-memory-bytes'
!

threadId
    "Return the thread id passed to --thread or nil
     if no --thread was given"

    | index |

    index := arguments indexOf: '--thread'.
    index == 0 ifTrue: [ ^ nil ].

    self assert: arguments size > index.
    self assert:(arguments at: index + 1) isInteger.

    ^ arguments at: index + 1

    "Created: / 18-11-2021 / 23:19:28 / Jan Vrany <jan.vrany@labware.com>"
! !

!GDBMI_data_write_memory_bytes methodsFor:'hooks'!

postExecuteIn: debugger result: result 
    result isDone ifTrue: [
        | event thread |

        thread := debugger threadForId: self threadId.   

        event := GDBMemoryChangedEvent new.
        event setThreadGroup: thread group.
        event setAddr: (arguments at: arguments size - 2).
        event setLen: (arguments last).

        debugger push: event.
    ].

    "Created: / 18-11-2021 / 23:24:22 / Jan Vrany <jan.vrany@labware.com>"
! !

