"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2021-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
"{ Package: 'jv:libgdbs' }"

"{ NameSpace: Smalltalk }"

GDBMICommand subclass:#GDBMI_stack_select_frame
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'GDB-Commands-MI'
!

!GDBMI_stack_select_frame class methodsFor:'documentation'!

copyright
"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2021-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
!

documentation
"
The `-stack-select-frame' Command
---------------------------------

Synopsis
........

      -stack-select-frame FRAMENUM

   Change the selected frame.  Select a different frame FRAMENUM on the
stack.

   This command in deprecated in favor of passing the `--frame' option
to every command.

{No value for `GDBN'} Command
.............................

The corresponding {No value for `GDBN'} commands are `frame', `up',
`down', `select-frame', `up-silent', and `down-silent'.

Example
.......

     (gdb)
     -stack-select-frame 2
     ^done
     (gdb)


"
! !

!GDBMI_stack_select_frame methodsFor:'accessing'!

operation
	^ 'stack-select-frame'
! !

!GDBMI_stack_select_frame methodsFor:'hooks'!

postExecuteIn: debugger result: result 
    result isDone ifTrue: [
        | threadArgIndex thread stack frame  event |

        threadArgIndex := arguments indexOf: '--thread'.
        threadArgIndex ~~ 0 ifTrue: [ 
            thread := debugger threadForId: (arguments at:threadArgIndex + 1).
        ] ifFalse: [
            thread := debugger selectedThread.
        ].

        stack := thread stackButNotStale.
        (stack notNil and: [ stack size > arguments last ]) ifTrue: [ 
            frame  := stack at: arguments last + 1    
        ] ifFalse: [ 
            frame := GDBFrame new 
                        setDebugger: debugger;
                        setThread: thread;
                        setLevel: arguments first;
                        yourself
        ].
        event := GDBThreadSelectedEvent new.
        event setThread: thread.
        event setFrame: frame.
        debugger push: event
    ].

    "Created: / 10-03-2021 / 12:55:22 / Jan Vrany <jan.vrany@labware.com>"
    "Modified: / 22-05-2021 / 17:17:43 / Jan Vrany <jan.vrany@labware.com>"
! !

!GDBMI_stack_select_frame class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

