"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2021-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
"{ Package: 'jv:libgdbs' }"

"{ NameSpace: Smalltalk }"

GDBMICommand subclass:#GDBMI_thread_select
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'GDB-Commands-MI'
!

!GDBMI_thread_select class methodsFor:'documentation'!

copyright
"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2021-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
!

documentation
"
The `-thread-select' Command
----------------------------

Synopsis
........

      -thread-select THREADNUM

   Make THREADNUM the current thread.  It prints the number of the new
current thread, and the topmost frame for that thread.

   This command is deprecated in favor of explicitly using the
`--thread' option to each command.

{No value for `GDBN'} Command
.............................

The corresponding {No value for `GDBN'} command is `thread'.

Example
.......

     (gdb)
     -exec-next
     ^running
     (gdb)
     *stopped,reason='end-stepping-range',thread-id='2',line='187',
     file='../../../devo/gdb/testsuite/gdb.threads/linux-dp.c'
     (gdb)
     -thread-list-ids
     ^done,
     thread-ids={thread-id='3',thread-id='2',thread-id='1'},
     number-of-threads='3'
     (gdb)
     -thread-select 3
     ^done,new-thread-id='3',
     frame={level='0',func='vprintf',
     args=[{name='format',value='0x8048e9c \'%*s%c %d %c\\n\''},
     {name='arg',value='0x2'}],file='vprintf.c',line='31'}
     (gdb)


"
! !

!GDBMI_thread_select methodsFor:'accessing'!

operation
	^ 'thread-select'
! !

!GDBMI_thread_select methodsFor:'accessing-descriptors'!

resultDescription     
    ^ (super resultDescription)
        define:#'frame' as: GDBFrame;
        define:#'new-thread-id' as: Integer;
        yourself

    "
    GDBMI_thread_select new resultDescription
    "

    "Created: / 09-03-2021 / 14:10:28 / Jan Vrany <jan.vrany@labware.com>"
! !

!GDBMI_thread_select methodsFor:'hooks'!

postExecuteIn: debugger result: result 
    result isDone ifTrue: [
        | event |

        event := GDBThreadSelectedEvent new.
        event 
            setThread: (debugger threadForId: (result propertyAt: 'new-thread-id')).
        event setFrame: (result propertyAt: 'frame').
        debugger push: event
    ].

    "Created: / 09-03-2021 / 14:17:05 / Jan Vrany <jan.vrany@labware.com>"
! !

