"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2022-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
"{ Package: 'jv:libgdbs' }"

"{ NameSpace: Smalltalk }"

GDBObject subclass:#GDBMemoryDump
	instanceVariableNames:'addr nr_bytes total_bytes next_row prev_row next_page prev_page
		memory'
	classVariableNames:''
	poolDictionaries:''
	category:'GDB'
!

!GDBMemoryDump class methodsFor:'documentation'!

copyright
"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2022-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
!

documentation
"
    GDBMemoryDump object represents a memory dump as produced by
    `x` or `-data-read-memory` commands.

    [author:]
        Jan Vrany <jan.vrany@fit.cvut.cz>

    [instance variables:]

    [class variables:]

    [see also:]
        GDBMI_data_read_memory
        https://sourceware.org/gdb/onlinedocs/gdb/GDB_002fMI-Data-Manipulation.html

"
! !

!GDBMemoryDump class methodsFor:'accessing-magritte'!

description
    ^ (super description)
        define:#addr as:String;
        define:#nr_bytes as:Integer;
        define:#total_bytes as:Integer;
        define:#memory as: Array of: GDBMemoryDumpRow;
        yourself

    "Created: / 25-01-2018 / 08:41:56 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 03-07-2018 / 17:06:37 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBMemoryDump methodsFor:'accessing'!

addr
    ^ addr
!

memory
    ^ memory
! !

!GDBMemoryDump methodsFor:'displaying'!

displayOn: aStream
    | addrW colW |

    addrW := addr size.
    colW := memory inject: 0 into:[:w1 :r | r data inject: w1 into: [ :w2 :val | w2 max: val size ] ].
    memory do:[:row|
        aStream nextPutAll: '0x'.
        row addr printOn: aStream base: 16 size: 16 fill: $0.
        aStream nextPut: $;; space.
        row data do:[:each | 
            aStream nextPutAll: each; next: colW - each size + 2 put: Character space.
        ].
        row ascii notNil ifTrue:[ 
            aStream space; nextPutAll: row ascii.
        ].
        aStream cr.
    ].

    "Created: / 25-01-2018 / 23:07:29 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 04-07-2018 / 14:00:49 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBMemoryDump methodsFor:'inspecting'!

inspector2TabCondition
    <inspector2Tab>

    ^ Tools::Inspector2Tab new
            priority: 90;
            label:'Memory';  
            text: [ self displayString ]
            yourself.

    "Created: / 25-01-2018 / 23:16:01 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

