"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2021-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
"{ Package: 'jv:libgdbs' }"

"{ NameSpace: Smalltalk }"

Object subclass:#GDBTransientDataHolder
	instanceVariableNames:'debugger factory value seqno lock'
	classVariableNames:'InvalidValue'
	poolDictionaries:''
	category:'GDB-Internal'
!

!GDBTransientDataHolder class methodsFor:'documentation'!

copyright
"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2021-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
! !

!GDBTransientDataHolder class methodsFor:'initialization'!

initialize
    "Invoked at system start or when the class is dynamically loaded."

    "/ please change as required (and remove this comment)

    InvalidValue := Object new

    "Modified: / 29-01-2018 / 23:58:57 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBTransientDataHolder class methodsFor:'instance creation'!

debugger: aGDBDebugger factory: aBlock
    ^ self new setDebugger: aGDBDebugger factory: aBlock

    "Created: / 17-09-2014 / 22:08:22 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBTransientDataHolder methodsFor:'accessing'!

value
    (value ~~ InvalidValue and:[debugger currentInferiorStateSequnceNumber == seqno]) ifTrue:[ 
        ^ value.
    ].
    lock critical:[
        (value ~~ InvalidValue and:[debugger currentInferiorStateSequnceNumber == seqno]) ifFalse:[
            | token old new |

            "/ The trick with `token` here is to avoid call to
            "/ #value: in case it has been called from within
            "/ the factory itself.
            old := value == InvalidValue ifTrue:[ nil ] ifFalse:[ value ].
            value := token := Object new.
            new := factory cull: old.
            value == token ifTrue:[ 
                self value: new.
            ].
        ].
    ].
    ^ value

    "Created: / 17-09-2014 / 22:06:58 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 26-09-2018 / 13:10:48 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (format): / 10-03-2021 / 14:34:22 / Jan Vrany <jan.vrany@labware.com>"
!

value: anObject
    value := anObject.
    seqno := debugger currentInferiorStateSequnceNumber.

    "Created: / 30-01-2018 / 00:05:06 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 30-01-2018 / 09:06:48 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

valueButNotStale
    "Return current value if it is up to date or nil if the value
     is stale and has not been updated. This method NEVER updates
     the value.
    "
    | retval |

    (value ~~ InvalidValue and:[debugger currentInferiorStateSequnceNumber == seqno]) ifTrue:[ 
        ^ value.
    ].
    lock critical:[
        (value ~~ InvalidValue and:[debugger currentInferiorStateSequnceNumber == seqno]) ifTrue:[
            retval := value
        ]
    ].
    ^ retval

    "Created: / 10-03-2021 / 14:50:58 / Jan Vrany <jan.vrany@labware.com>"
!

valuePossiblyStale
    "Return current value, no matter whether its (possibly) stale or not.
     In other words, this method does not check nor update the value
     (if needed).
    "
    ^ value == InvalidValue ifTrue:[ nil ] ifFalse:[ value ]

    "Created: / 10-03-2021 / 14:36:03 / Jan Vrany <jan.vrany@labware.com>"
! !

!GDBTransientDataHolder methodsFor:'initialization'!

setDebugger: aGDBDebugger factory: aBlock
    debugger := aGDBDebugger.
    factory := aBlock.
    value := InvalidValue.
    lock := RecursionLock new.

    "Created: / 17-09-2014 / 22:08:47 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 08-09-2018 / 15:40:30 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBTransientDataHolder methodsFor:'utilities'!

invalidate
    value := InvalidValue

    "Created: / 10-09-2021 / 15:34:40 / Jan Vrany <jan.vrany@labware.com>"
! !

!GDBTransientDataHolder class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !


GDBTransientDataHolder initialize!
