"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2020 LabWare
Copyright (C) 2022-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
"{ Package: 'jv:libgdbs' }"

"{ NameSpace: Smalltalk }"

GDBLocalProcess subclass:#GDBUnixProcess
	instanceVariableNames:'debuggerPTY consolePTY'
	classVariableNames:''
	poolDictionaries:''
	category:'GDB-Internal'
!

!GDBUnixProcess class methodsFor:'documentation'!

copyright
"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2020 LabWare
Copyright (C) 2022-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
!

documentation
"
    GDBStXUnixProcess is a Smalltalk/X specific implementation of GDB process
    that uses two PTYs to communicate with GDB - one for MI channel, another 
    for CLI channel. In this case, the CLI interface is driven by GDB itself
    (as opposite to be emulated as in case of GDBStXSimpleProcess).

    On UNIX platforms running GDB locally, this is preferred implementation
    to use.

    [author:]
        Jan Vrany <jan.vrany@fit.cvut.cz>

    [instance variables:]

    [class variables:]

    [see also:]
        GDBRemoteProcess
        GDBStXSimpleProcess

"
! !

!GDBUnixProcess methodsFor:'accessing'!

consoleInput
    ^ consolePTY masterWriteStream

    "Created: / 02-06-2017 / 23:36:24 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 02-08-2023 / 12:50:52 / Jan Vrany <jan.vrany@labware.com>"
!

consoleOutput
    ^ consolePTY masterReadStream

    "Created: / 02-06-2017 / 23:36:35 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 02-08-2023 / 12:50:41 / Jan Vrany <jan.vrany@labware.com>"
!

nativeTargetFeatures
    ^ #('async')

    "Created: / 09-04-2018 / 15:40:32 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBUnixProcess methodsFor:'initialization & release'!

initializeWithCommand: aString
    | conpty dbgpty argv |

    conpty := GDBPortlib current newPTY.
    dbgpty := GDBPortlib current newPTY.
    dbgpty setLocalEcho: false.
    dbgpty setOutputCRLF: false.   

    command := aString.
    argv := ((self command2argv: command)) ,
            ((Array new: 5)
                at: 1 put: '-q';
                at: 2 put: '-iex';
                at: 3 put: 'new-ui mi ', dbgpty name;
                at: 4 put: '-iex';
                at: 5 put: 'set pagination off';
                yourself).

    pid := GDBPortlib current 
                 spawn: argv
                 stdin: conpty slaveReadStream
                 stdout: conpty slaveWriteStream
                 stderr: conpty slaveWriteStream
                 exit: [:stat | self exited: stat ].             
    pid isNil ifTrue:[
        conpty close.
        dbgpty close.
        self error:'Failed to launch gdb'.
    ].

    consolePTY := conpty.
    debuggerPTY := dbgpty.
    debuggerInput := dbgpty masterWriteStream. 
    debuggerOutput := dbgpty masterReadStream.

    "Created: / 12-12-2018 / 20:13:17 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 11-06-2019 / 11:44:56 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 02-08-2023 / 13:07:17 / Jan Vrany <jan.vrany@labware.com>"
!

release
    debuggerPTY notNil ifTrue:[
        debuggerPTY release.
        debuggerInput := debuggerOutput := nil
    ].
    consolePTY notNil ifTrue:[
        consolePTY release.
    ].
    super release

    "Created: / 02-06-2017 / 23:33:47 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 15-12-2017 / 23:59:56 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBUnixProcess methodsFor:'queries'!

canUsePTY
    "Return true if PTYs can be used with this process, false otherwise"

    ^ true

    "Created: / 16-01-2019 / 23:02:20 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBUnixProcess class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

